#
# ContentContainer product
# Peter Bengtsson <peter@galdrion.com>
# Start date: 3/01 -02
# 
# HISTORY
#
# 0.1  2002-01-03 Built from scratch
#


__version__='0.1'
__doc__="""Simple content container.
Content Container was built to hold two simple properties, "content1" and "content2" and being
able to product different resulting content depending on language.

This class subclasses the simple DTMLDocument class, but can NOT serve DTML! 
This is something that could be a cool feature, but not necessary and would 
complicate things.

The class overrides the getProperty method from the baseclass and in this
version it looks at the language, then decides which "version" of the 
property to serve.

The class does not have a 'title' property; so one must use getTitle(), 
title_or_id() or getProperty('title')

The __str__/__call__ method is very simple. It returns the correct content
text property passed though little text parsors found in the Utils.
If the appropirate content is null, a default message ("Content not translated") 
is returned."""

# python
import string
from urllib import quote

# Zope
from OFS.DTMLDocument import DTMLDocument
from Globals import MessageDialog, Persistent, DTMLFile, default__class_init__
from AccessControl import ClassSecurityInfo
from Products.ZCatalog.CatalogAwareness import CatalogAware
from OFS.SimpleItem import SimpleItem

# product
from ContentContainerUtils import ST, valid_format, PlainText, highlight_text



manage_addContentContainerForm = DTMLFile('dtml/addContentContainerForm', globals())

def addContentContainer(self, id, title1='', title2='', language1='sv', language2='en', REQUEST=None, submit=None):
    """Add a ContentContanier object with the contents of file. If
    'file' is empty, default document text is used.
    """
    id=str(id).strip()
    title1=str(title1).strip()
    title2=str(title2).strip()

    if language1 == '' or language1 is None:
	language1 = 'sv'

    if title2=='' and title1!='':
	title2 = title1

    ob=ContentContainer(id, title1=title1, title2=title2, language1=language1, language2=language2)

    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''

manage_addContentContainer = addContentContainer


class ContentContainer(SimpleItem, DTMLDocument, CatalogAware):
    """ ContentContainer class """
    
    meta_type = 'Content Container'
    
    manage_options = ({'label':'Edit1', 'action':'Edit1'},
		      {'label':'Edit2', 'action':'Edit2'})+DTMLDocument.manage_options[1:]

    _properties=(#{'id':'title', 'type': 'string', 'mode': 'w'},
                 {'id':'format', 'type': 'string', 'mode': 'w'},
                 {'id':'default_language', 'type': 'string', 'mode': 'w'},
                )
		      
    security=ClassSecurityInfo()
    
    def __init__(self, id, 
		title1='', content1='', language1='sv',
		title2='', content2='', language2='en', 
		default_language='sv',  format='structuredtext',
		REQUEST=None):
	""" init ContentContainer """
	
	self.id = id
	self.title1 = title1
	self.content1 = content1
	self.language1 = self.valid_language(language1)
	self.title2 = title2
	self.content2 = content2
	self.language2 = self.valid_language(language2)
	self.format = format
#	self.title = self.get_title()
	self.default_language = self.valid_language(default_language)

	
    Edit1 = DTMLFile('dtml/documentEdit1', globals())
    Edit2 = DTMLFile('dtml/documentEdit2', globals())
    manage_main = Edit1
    
    security.declareProtected('Access contents information', 'get_property')
    def get_property(self, property):
	""" return correct property depending on language """
	lang = self.get_language()
	no = self.language2no(lang)

	try:
	    p= getattr(self, '%s%s'%(property, no))
	    return p
	except:
	    p= getattr(self, property)
	    return p

    security.declareProtected('Access contents information', 'absolute_url')
    def absolute_url(self, relative=0):
	""" This is a hack to the baseclass's absolute_url() method.
	    For some reason the baseclass absolute_url() returns 'http://host/folder/ElementWithAttributes'
	    instead of 'http://host/folder/<objectid>' for some bizarre reason.
	    This hack compensates for VirtualHosts.
	"""
	
	try:
	    req = self.REQUEST
	except AttributeError:
	    req = {}
	rpp = req.get('VirtualRootPhysicalPath', ('',))
	spp = self.getPhysicalPath()
	i = 0
	for name in rpp[:len(spp)]:
	    if spp[i] == name:
		i = i + 1
	    else:
		break
	path = map(quote, spp[i:])
	if relative:
	    # This is useful for physical path relative to a VirtualRoot
	    return self._fix_absolute_url(string.join(path, '/'))
	return self._fix_absolute_url(string.join([req['SERVER_URL']] + req._script + path, '/'))
    
    def _fix_absolute_url(self, url):
	""" fix the last bit """
	bits = url.split('/')
	bits = bits[:-1]+[self.id]
	return string.join(bits, '/')
   
    security.declareProtected('Access contents information', 'language2no')
    def language2no(self, l):
	""" return which numbered property to use depending on language """
	if self.language2 == l:
	    return 2
	else:
	    # default is 1
	    return 1

    security.declareProtected('Manage properties', 'manage_edit')
    def manage_edit(self, property_no, 
                    title=None, content=None ,language=None,
		    SUBMIT='Change',dtpref_cols='50',
                    dtpref_rows='20',REQUEST=None):
        """
	edit a ContentContainer
        """
        self._validateProxy(REQUEST)
        if self.wl_isLocked():
            raise ResourceLockedError, 'This Content Container is locked via WebDAV'
	
	property_no = int(property_no)
	v_lang = self.valid_language(language)
	
	if property_no==1:
	    self.title1=str(title)
	    self.content1 = str(content)
	    if v_lang is not None:
		self.language1 = v_lang
	elif property_no==2:
	    self.title2 = str(title)
	    self.content2 = str(content)
	    if v_lang is not None:
		self.language2 = v_lang
        self.ZCacheable_invalidate()
        if REQUEST:
            message="Saved changes."
	    if property_no==1:
		return self.Edit1(self,REQUEST,manage_tabs_message=message)
	    elif property_no==2:
		return self.Edit2(self,REQUEST,manage_tabs_message=message)

    
    security.declareProtected('Access contents information', 'valid_language')
    def valid_language(self, lang):
	""" return lang or None if the lang is not recognized """
	translators = {'svenska':'sv', 'svensk':'sv', 'swedish':'sv',
	               'se':'sv', 'english':'en', 'engelska':'en'}
	lang = str(lang.strip().lower())
	try:
	    return translators[lang]
	except:
	    pass
	
	# more tests ?
	return lang

    security.declareProtected('Access contents information', 'get_language')
    def get_language(self):
	""" return language ISO code """
	# expect 'sv' or 'en' or 'whatever'
	
	debug = 1
	
	# HTTP test
	http_language = self.REQUEST.get('HTTP_ACCEPT_LANGUAGE',self.default_language)

	# if method exists already somewhere else for getting the ISO code
	if hasattr(self, 'getLanguage'):
	    http_language = self.getLanguage()

	# debug test
	if debug:
	    if self.REQUEST.has_key('debug_lang'):
		http_language = self.REQUEST['debug_lang']
		
	# cookie/session test
	# not done
	
	if self.REQUEST.has_key('Language'):
	    return self.REQUEST['Language']
	
	return http_language
	
    security.declareProtected('Access contents information', 'get_title')
    def get_title(self):
	""" get the right title depending on language """
	return self.get_property('title')
    	
    security.declareProtected('Access contents information', 'isTranslatedTo')
    def isTranslatedTo(self, lang):
	""" return if object is translated to requested language or not. It is not if the object's title AND content is null """
	no = self.language2no(lang)
	if self.get_property('title')=='' and self.get_property('content')=='':
	    return 0
	else:
	    return 1
	
    security.declareProtected('Access contents information', 'title_or_id')
    def title_or_id(self):
	""" return title_or_id based on language """
	return self.get_property('title') or self.id
    
    security.declareProtected('View', 'RenderContent')
    def RenderContent(self, REQUEST=None, client=None):
	content = self.get_property('content')
	if content.strip()=='':
	    return "Content not translated"
	if valid_format(self.format) == 'structuredtext':
	    content= self._highlight_text(content)
	    s= ST(content)
	elif valid_format(self.format) == 'plaintext':
	    s= PlainText(content)
	    s= self._highlight_text(s)
	else:
	    s= content
	    s= self._highlight_text(s)
	return s
	
    def _highlight_text(self, s):
	""" highlight words in text """
	req = self.REQUEST
	if req.has_key('q-rp'):
	    if hasattr(self, 'highlight_starttag') and hasattr(self, 'highlight_endtag'):
		starttag = self.highlight_starttag
		endtag = self.highlight_endtag
	    else:
		starttag = '<strong style="background:#CCFF99;">'
		endtag = '</strong>'
	    s = highlight_text(s, self.REQUEST['q-rp'], starttag, endtag)
	return s

    
    # Aliases
    __call__=RenderBody=__str__=RenderContent
    getProperty = get_property
    getTitle = get_title
    
    
    
default__class_init__(DTMLDocument)
